(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('three-full')) :
    typeof define === 'function' && define.amd ? define(['three-full'], factory) :
    (global.THREEViewCubeObject3D = factory(global.THREE));
}(this, (function (threeFull) { 'use strict';

    threeFull = threeFull && threeFull.hasOwnProperty('default') ? threeFull['default'] : threeFull;

    /**
     * @author maximequiblier
     *
     * @param {Object} materials
     *      @param {THREE.Material} materials.front The front material
     *      @param {THREE.Material} materials.back The back material
     *      @param {THREE.Material} materials.left The left material
     *      @param {THREE.Material} materials.right The right material
     *      @param {THREE.Material} materials.top The top material
     *      @param {THREE.Material} materials.bottom The bottom material
     * @param {Object} hover Custom parameters for hover
     *      @param {THREE.Color|Number|String} color The material color of the hover element will be replaced with this color on hover
     *      @param {THREE.Color|Number|String} outlineColor The outline color. If null, no outline, if undefined, a black outline.
     *
     */
    function ViewCubeObject3D(params) {

        threeFull.Object3D.call( this );

        var params = params || {};

        this.name = params.name || "";

        var frontg = new threeFull.PlaneBufferGeometry();
        frontg.translate(0,0,0.5);
        var backg = frontg.clone();
        backg.rotateY(Math.PI);

        var rightg = frontg.clone();
        rightg.rotateY(Math.PI/2);
        var leftg = frontg.clone();
        leftg.rotateY(-Math.PI/2);

        var topg = frontg.clone();
        topg.rotateX(-Math.PI/2);
        var botg = frontg.clone();
        botg.rotateX(Math.PI/2);

        var frontm = new threeFull.Mesh(
            frontg,
            params.materials && params.materials.front ? params.materials.front : new threeFull.MeshBasicMaterial({ color: 0x0000ff })
        );
        frontm.name = "front";
        var backm = new threeFull.Mesh(
            backg,
            params.materials && params.materials.back ? params.materials.back : new threeFull.MeshBasicMaterial({ color: 0x0000ff })
        );
        backm.name = "back";

        var leftm = new threeFull.Mesh(
            leftg,
            params.materials && params.materials.left ? params.materials.left : new threeFull.MeshBasicMaterial({ color: 0xff0000 })
        );
        leftm.name = "left";
        var rightm = new threeFull.Mesh(
            rightg,
            params.materials && params.materials.right ? params.materials.right : new threeFull.MeshBasicMaterial({ color: 0xff0000 })
        );
        rightm.name = "right";

        var topm = new threeFull.Mesh(
            topg,
            params.materials && params.materials.top ? params.materials.top : new threeFull.MeshBasicMaterial({ color: 0x00ff00 })
        );
        topm.name = "top";
        var botm = new threeFull.Mesh(
            botg,
            params.materials && params.materials.bottom ? params.materials.bottom : new threeFull.MeshBasicMaterial({ color: 0x00ff00 })
        );
        botm.name = "bottom";

        this.add(frontm);
        this.add(backm);
        this.add(rightm);
        this.add(leftm);
        this.add(topm);
        this.add(botm);

        this.hoverMesh = null;
        this.hoverOutline = null;
        if (params.hover && params.hover.outlineColor !== null){
            this.hoverOutline = new threeFull.LineSegments(
                null,
                new threeFull.LineBasicMaterial({ color: params.hover.outlineColor !== undefined ? params.hover.outlineColor : 0x000000 })
            );
        }

        this.hoverColor = params.hover && params.hover.color ? new threeFull.Color(params.hover.color) : new threeFull.Color(0x00FFFF);
        this.hoverPrevColor = null;
    }
    ViewCubeObject3D.prototype = Object.create( threeFull.Object3D.prototype );
    ViewCubeObject3D.prototype.constructor = ViewCubeObject3D;

    ViewCubeObject3D.prototype.getWorldOrientations = function(){
        var res = {
            front:new threeFull.Vector3(0,0,0),
            back: new threeFull.Vector3(Math.PI, 0, -Math.PI),
            left: new threeFull.Vector3(0, -Math.PI/2, 0),
            right: new threeFull.Vector3(0, Math.PI/2, 0),
            top: new threeFull.Vector3(-Math.PI/2, 0, 0),
            bottom: new threeFull.Vector3(Math.PI/2, 0, 0)
        };
        this.updateMatrixWorld();
        var q = this.getWorldQuaternion(new threeFull.Quaternion());
        Object.keys(res).forEach(function(k){
            var v = res[k];
            v.applyQuaternion(q);
            res[k] = new threeFull.Euler(v.x,v.y,v.z);
        });
        return res;
    };

    ViewCubeObject3D.prototype.hover = function(name){
        if(this.hoverMesh){
            this.hoverMesh.material.color = this.hoverPrevColor;
            this.hoverPrevColor = null;
            this.hoverMesh = null;
            if (this.hoverOutline){
                this.hoverOutline.geometry.dispose();
                this.hoverOutline.parent.remove(this.hoverOutline);
            }
        }
        if(name){
            var o = this.getObjectByName(name);
            if(o){
                this.hoverMesh = o;
                this.hoverPrevColor = this.hoverMesh.material.color;
                this.hoverMesh.material.color = this.hoverColor;
                if (this.hoverOutline){
                    this.hoverOutline.geometry = new threeFull.EdgesGeometry(this.hoverMesh.geometry);
                    this.hoverMesh.add(this.hoverOutline);
                }
            }
        }
    };

    var ViewCubeObject3D_1 = ViewCubeObject3D;

    threeFull.ViewCubeObject3D = ViewCubeObject3D_1;

    var exports$1 = ViewCubeObject3D_1;

    return exports$1;

})));
